#include <iostream>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>

#include "lw14.h"

using namespace std;


int scan_i2c_bus(int);
int lw14_Send(int, int, int);
int lw14_NewI2C(int, int);
int lw14_SetBroadcastAddress(unsigned char);
int lw14_SetGroupAddress(int, unsigned char);
int lw14_SetShortAddress(int, unsigned char);


int scan_i2c_bus(int device)
{
    int port, res, numDev = 0;

    printf("Scan for decices....\n\n");

    for(port = 0; port < 127; port++)
    {
        if(ioctl(device, I2C_SLAVE, port) < 0)
            perror("ioctl() I2C_SLAVE failed\n");
        else
        {
            res = i2c_smbus_read_byte(device);
            if(res >= 0)
            {
                printf("i2c device found at: 0x%02X, val = %d\n", port, res);
                numDev++;
            }
        }
    }

    return numDev;
}


int lw14_SetBroadcastAddress(unsigned char mode)
{
	return (0xFE | mode);
}

int lw14_SetGroupAddress(int group, unsigned char mode)
{
	return (0x80 | ((group & 15) << 1) | mode);
}

int lw14_SetShortAddress(int device, unsigned char mode)
{
	return (((device & 63) << 1) | mode);
}

int lw14_Send(int device, int adr, int value)
{
    int res = 0;
    unsigned char data[] = {(unsigned char) adr, (unsigned char) value};

    if(ioctl(device, I2C_SLAVE, LW14_I2CADR) >= 0)
    {
        //device handle, register, datacount, datas
        res = i2c_smbus_write_i2c_block_data(device, 0x01, 2, &data[0]) ;

        if(res < 0)
            printf("Something went wrong when write. Maybe wrong I2C-Address?\n");
    }
    else
        printf("Error when write. ioctl() I2C not found\n");

    return res;
}

int lw14_NewI2C(int device, int adr)
{
    int res = 0;
    //negated addr value to check by the chip
    unsigned char data[] = {(unsigned char) adr, (unsigned char) ~(adr)};

    if(ioctl(device, I2C_SLAVE, LW14_I2CADR) >= 0)
    {
        res = i2c_smbus_write_i2c_block_data(device, 0xFE, 2, &data[0]);

        if(res < 0)
            printf("Something went wrong when write. Maybe wrong I2C-Address?\n");
    }

    else
        printf("Error when write. ioctl() I2C not found\n");

    return res;
}

int main(int argc, char** argv)
{

    int device;
    unsigned long funcs;
    int dali_adr;
    int numDev = 0;

    //Open I2c interface
    if((device = open("/dev/i2c-1", O_RDWR)) < 0)
    {
        perror("Failed to open I2C bus\n");
        exit(1);
    }


    if(ioctl(device, I2C_FUNCS, &funcs) < 0)
    {
        perror("ioctl() I2C_FUNCS failed\n");
        exit(1);
    }


    //started program without parameter
    if(argc == 1)
    {
        system ("echo \"$(cat help.txt)\"");
    }

    if((argc >= 2) && strcmp(argv[1], "-l") == 0)
    {
        numDev = scan_i2c_bus(device);
        printf("Found %d I2C devices\n", numDev);
    }

    //Send command to all DALI devices (BROADCAST)
    if((argc >= 2) && strcmp(argv[1], "-b") == 0)
    {
        //./lw14cmd -b X X
        if(argc == 4)
        {
            printf("Send BROADCAST command\n");

      		dali_adr = lw14_SetBroadcastAddress(atoi(argv[2]));
            lw14_Send(device, dali_adr, atoi(argv[3]));
        }
        else
            printf("Input error. Please check input\n");
    }

    //Send command to a DALI group (GROUP)
    if((argc >= 2) && strcmp(argv[1], "-g") == 0)
    {
        //./lw14cmd -g X X X
        if(argc == 5)
        {
            printf("Send GROUP command\n");

            dali_adr = lw14_SetGroupAddress(atoi(argv[2]), atoi(argv[3]));
            lw14_Send(device, dali_adr, atoi(argv[4]));
        }
        else
            printf("Input error. Please check input\n");
    }

    //Send command to a single DALI device (SHORT)
    if((argc >= 2) && strcmp(argv[1], "-d") == 0)
    {
        //./lw14cmd -d X X X
        if(argc == 5)
        {
            printf("Send SHORT command\n");

            dali_adr = lw14_SetShortAddress(atoi(argv[2]), atoi(argv[3]));
            lw14_Send(device, dali_adr, atoi(argv[4]));
        }
        else
            printf("Input error. Please check input\n");
    }

    //Change I2C-Address of device
    if((argc >= 2) && strcmp(argv[1], "-i") == 0)
    {
        //./lw14cmd -i X
        if(argc == 3)
        {
            printf("Set new I2C-Address\n");

            lw14_NewI2C(device, atoi(argv[2]));
        }
        else
            printf("Input error. Please check input\n");
    }

    close(device);

    return 0;
}
